package dev.fyloz.colorrecipesexplorer.logic.jobs

import dev.fyloz.colorrecipesexplorer.logic.TouchUpKitLogic
import mu.KotlinLogging
import org.springframework.context.annotation.Profile
import org.springframework.scheduling.annotation.Scheduled
import org.springframework.stereotype.Component

@Component
@Profile("!emergency")
class TouchUpKitRemover(
    private val touchUpKitLogic: TouchUpKitLogic
) {
    private val logger = KotlinLogging.logger {}

    @Scheduled(cron = "0 0 0 * * *")
    fun execute() {
        logger.debug("Executing expired touch up kits removal job... ")
        removeExpiredKits()
    }

    private fun removeExpiredKits() {
        with(touchUpKitLogic.getAll().filter(touchUpKitLogic::isExpired)) {
            this.forEach {
                logger.debug("Removed expired touch up kit ${it.id} (${it.project} ${it.buggy})")
                touchUpKitLogic.delete(it)
            }
            logger.info("Removed ${this.size} expired touch up kits")
        }
    }
}
