package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewCatalog
import dev.fyloz.colorrecipesexplorer.model.Company
import dev.fyloz.colorrecipesexplorer.model.CompanySaveDto
import dev.fyloz.colorrecipesexplorer.model.CompanyUpdateDto
import org.springframework.context.annotation.Profile
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

private const val COMPANY_CONTROLLER_PATH = "api/company"

@RestController
@RequestMapping(COMPANY_CONTROLLER_PATH)
@Profile("!emergency")
@PreAuthorizeViewCatalog
class CompanyController(private val companyLogic: dev.fyloz.colorrecipesexplorer.logic.CompanyLogic) {
    @GetMapping
    fun getAll() =
      ok(companyLogic.getAllForOutput())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
      ok(companyLogic.getByIdForOutput(id))

    @PostMapping
    @PreAuthorize("hasAuthority('EDIT_COMPANIES')")
    fun save(@Valid @RequestBody company: CompanySaveDto) =
      created<Company>(COMPANY_CONTROLLER_PATH) {
          companyLogic.save(company)
      }

    @PutMapping
    @PreAuthorize("hasAuthority('EDIT_COMPANIES')")
    fun update(@Valid @RequestBody company: CompanyUpdateDto) =
      noContent {
          companyLogic.update(company)
      }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('EDIT_COMPANIES')")
    fun deleteById(@PathVariable id: Long) =
      noContent {
          companyLogic.deleteById(id)
      }
}
