package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewCatalog
import dev.fyloz.colorrecipesexplorer.logic.MaterialLogic
import dev.fyloz.colorrecipesexplorer.model.*
import org.springframework.context.annotation.Profile
import org.springframework.http.MediaType
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile
import javax.validation.Valid

private const val MATERIAL_CONTROLLER_PATH = "api/material"

@RestController
@RequestMapping(MATERIAL_CONTROLLER_PATH)
@Profile("!emergency")
@PreAuthorizeViewCatalog
class MaterialController(
    private val materialLogic: MaterialLogic
) {
    @GetMapping
    fun getAll() =
        ok(materialLogic.getAllForOutput())

    @GetMapping("notmixtype")
    fun getAllNotMixType() =
        ok(materialLogic.getAllNotMixType())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(materialLogic.getByIdForOutput(id))

    @PostMapping(consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorize("hasAuthority('EDIT_MATERIALS')")
    fun save(@Valid material: MaterialSaveDto, simdutFile: MultipartFile?) =
        created<MaterialOutputDto>(MATERIAL_CONTROLLER_PATH) {
            with(materialLogic) {
                save(
                    materialSaveDto(
                        name = material.name,
                        inventoryQuantity = material.inventoryQuantity,
                        materialTypeId = material.materialTypeId,
                        simdutFile = simdutFile
                    )
                ).toOutput()
            }
        }

    @PutMapping(consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorize("hasAuthority('EDIT_MATERIALS')")
    fun update(@Valid material: MaterialUpdateDto, simdutFile: MultipartFile?) =
        noContent {
            materialLogic.update(
                materialUpdateDto(
                    id = material.id,
                    name = material.name,
                    inventoryQuantity = material.inventoryQuantity,
                    materialTypeId = material.materialTypeId,
                    simdutFile = simdutFile
                )
            )
        }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('EDIT_MATERIALS')")
    fun deleteById(@PathVariable id: Long) =
        noContent {
            materialLogic.deleteById(id)
        }

    @GetMapping("mix/create/{recipeId}")
    fun getAllForMixCreation(@PathVariable recipeId: Long) =
        ok(materialLogic.getAllForMixCreation(recipeId))

    @GetMapping("mix/update/{mixId}")
    fun getAllForMixUpdate(@PathVariable mixId: Long) =
        ok(materialLogic.getAllForMixUpdate(mixId))
}

