package dev.fyloz.colorrecipesexplorer.logic

import com.nhaarman.mockitokotlin2.*
import dev.fyloz.colorrecipesexplorer.model.*
import dev.fyloz.colorrecipesexplorer.repository.CompanyRepository
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import kotlin.test.assertFalse
import kotlin.test.assertTrue

@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class CompanyLogicTest :
  AbstractExternalNamedModelServiceTest<Company, CompanySaveDto, CompanyUpdateDto, CompanyLogic, CompanyRepository>() {
    private val recipeLogic: RecipeLogic = mock()
    override val repository: CompanyRepository = mock()
    override val logic: CompanyLogic = spy(
        DefaultCompanyLogic(
            repository,
            recipeLogic
        )
    )

    override val entity: Company = company(id = 0L, name = "company")
    override val anotherEntity: Company = company(id = 1L, name = "another company")
    override val entityWithEntityName: Company = company(id = 2L, name = entity.name)
    override val entitySaveDto: CompanySaveDto = spy(companySaveDto())
    override val entityUpdateDto: CompanyUpdateDto = spy(companyUpdateDto(id = entity.id!!, name = null))

    @AfterEach
    override fun afterEach() {
        reset(recipeLogic)
        super.afterEach()
    }

    // isLinkedToRecipes

    @Test
    fun `isLinkedToRecipes() returns true when a given company is linked to one or more recipes`() {
        whenever(recipeLogic.existsByCompany(entity)).doReturn(true)

        val found = logic.isLinkedToRecipes(entity)

        assertTrue(found)
    }

    @Test
    fun `isLinkedToRecipes() returns false when a given company is not linked to any recipe`() {
        whenever(recipeLogic.existsByCompany(entity)).doReturn(false)

        val found = logic.isLinkedToRecipes(entity)

        assertFalse(found)
    }

    // save()

    @Test
    override fun `save(dto) calls and returns save() with the created entity`() {
        withBaseSaveDtoTest(entity, entitySaveDto, logic)
    }

    // update()

    @Test
    override fun `update(dto) calls and returns update() with the created entity`() =
      withBaseUpdateDtoTest(entity, entityUpdateDto, logic, { any() })

    // delete()

    override fun `delete() deletes in the repository`() {
        whenCanBeDeleted {
            super.`delete() deletes in the repository`()
        }
    }

    // deleteById()

    override fun `deleteById() deletes the entity with the given id in the repository`() {
        whenCanBeDeleted {
            super.`deleteById() deletes the entity with the given id in the repository`()
        }
    }

    private fun whenCanBeDeleted(id: Long = any(), test: () -> Unit) {
        whenever(repository.canBeDeleted(id)).doReturn(true)

        test()
    }
}
